import sys
import os
import math

import conf

no_keyword_mode = False

def run_cmd(*args):
	pid = os.fork()
	if(pid == 0):
		os.execvp(args[0], args)
	#pid, status = os.waitpid(pid, 0)

def cos_sim(cmd1, cmd2):
	word_set = set(cmd1.split()) | set(cmd2.split())
	num = 0; den1 = 0; den2 = 0
	for word in word_set:
		count1 = cmd1.count(word)
		count2 = cmd2.count(word)
		num = num + count1 * count2
		den1 = den1 + count1 * count1
		den2 = den2 + count2 * count2
	den = math.sqrt(den1) * math.sqrt(den2)
	if(den == 0):
		return 0
	return num / den

# Looks for splitting keywords
# Returns an array with a single or multiple commands
def split_cmd(cmd):
	words = cmd.split()
	cmd_list = []
	last_i = 0
	# Loop through words to find splitting keywords
	for i, word in enumerate(words):
		if word in conf.chain:
			s_cmd = " ".join(words[last_i:i])
			if s_cmd != "":
				cmd_list.append(s_cmd)
			last_i = i + 1
	# No chaining - return the command
	if last_i == 0:
		return [cmd]
	# Get the last command
	last_cmd = " ".join(words[last_i:len(words)])
	cmd_list.append(last_cmd)
	return cmd_list


# Separate numbers from command
def strip_cmd(cmd):
	words = cmd.split()
	args = []
	for i, word in enumerate(words):
		if word in conf.numbers:
			args.append(words[i])
			words[i] = "#"
	return " ".join(words), args


def find_cmd(in_cmd):
	tgt_sim = 0; tgt_cmd = None; tgt_fnd = False
	clean_cmd, cmd_args = strip_cmd(in_cmd)
	for cmd in conf.commands:
		sim = cos_sim(clean_cmd, cmd)
		if sim > tgt_sim:
			tgt_sim = sim
			tgt_cmd = cmd
			tgt_fnd = False
		elif sim == tgt_sim:
			tgt_fnd = True
	# If 2 commands have the same sim, return None
	if tgt_fnd:
		tgt_cmd = None
	return tgt_cmd, tgt_sim, cmd_args

def insert_cmd_args(cmd, cmd_args):
	words = cmd.split()
	arg_index = 0
	for i, word in enumerate(words):
		if word == "#":
			words[i] = cmd_args[arg_index]
			arg_index += 1
	return " ".join(words)

def process_cmd(cmd):
	cmd, sim, cmd_args = find_cmd(cmd)
	print(f"FOUND: {cmd} (sim={sim:.2f})")
	if cmd == None:
		return
	global no_keyword_mode
	# Get value associated with command key
	action = conf.commands.get(cmd)
	# Re-add numeric arguments to command
	cmd = insert_cmd_args(cmd, cmd_args)
	if isinstance(action, list):
		run_cmd(*action)
	elif action == "keyboard":
		run_cmd("python.exe", "C:\\dev\\uni\\dooku\\kb.py", cmd)
	elif action == "continuous_enable":
		print("Continuous listening ON!")
		no_keyword_mode = True
	elif action == "continuous_disable":
		print("Continuous listening OFF!")
		no_keyword_mode = False
	elif action == "exit":
		exit()

def check_keyword(cmd):
	words = cmd.split()
	# If keyword found return the command without it
	if len(words) > 0 and words[0] in conf.keywords:
		return " ".join(words[1:])
	elif no_keyword_mode:
		return cmd
	else:
		return None

def main_loop():
	for cmd in sys.stdin:
		# Clean input
		cmd = cmd.rstrip('\r\n')
		print("COMMAND:", cmd)
		# Check if keyword was spoken
		cmd = check_keyword(cmd)
		if cmd != None:
			cmd_list = split_cmd(cmd)
			#print("cmd_list", cmd_list)
			for cmd in cmd_list:
				process_cmd(cmd)

# Start the program
main_loop()
